
/*
 * ServerThreadPool.java
 *
 * Created on 10 de Maio de 2006, 11:12
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package br.ufsc.das.blocking;

import br.ufsc.das.util.EventListener;
import br.ufsc.das.util.ReceivedMessage;
import java.io.DataInputStream;
import java.net.Socket;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.ThreadFactory;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

/**
 *
 * @author edualchieri
 */
public class ServerThreadPool extends ThreadPoolExecutor{
    private EventListener listener;
    private ConnectionManager manager;
   
    
    
    /** Creates a new instance of ServerThreadPool */
    public ServerThreadPool(int corePoolSize,
            int maximumPoolSize,
            long keepAliveTime,
            TimeUnit unit,
            BlockingQueue<Runnable> workQueue,
            EventListener listener,
            ConnectionManager manager) {
        super(corePoolSize,maximumPoolSize,keepAliveTime,unit,workQueue);
        this.manager = manager;
        this.listener = listener;
        this.setThreadFactory(new ServerThreadFactory());
    }
    
    
    public void execute(DataInputStream in, int from, SynchronizedOutput so){
        //br.ufsc.das.util.Logger.println("(ServerThreadPool) Threads na pool:"+this.getPoolSize());
        //br.ufsc.das.util.Logger.println("(ServerThreadPool) Threads em execucao:"+this.getActiveCount());
        //br.ufsc.das.util.Logger.println("(ServerThreadPool) Pedidos na fila:"+this.getQueue().size());
        this.execute(new ServerRunnable(in,from,so,this));
    }
    
    public void execute(Socket s){
        //br.ufsc.das.util.Logger.println("(ServerThreadPool) Threads na pool:"+this.getPoolSize());
        //br.ufsc.das.util.Logger.println("(ServerThreadPool) Threads em execucao:"+this.getActiveCount());
        //br.ufsc.das.util.Logger.println("(ServerThreadPool) Pedidos na fila:"+this.getQueue().size());
        this.execute(new ServerRunnable(s,this));
    }
    
    
    public class ServerRunnable implements Runnable{
        private Socket s;
        private DataInputStream in;
        private int from;
        private ServerThreadPool stp;
        private SynchronizedOutput so;
        
        public ServerRunnable(Socket s, ServerThreadPool stp) {
            this.s = s;
            this.stp = stp;
        }
        
        public ServerRunnable(DataInputStream in, int from, SynchronizedOutput so, ServerThreadPool stp) {
            this.in = in;
            this.from = from;
            this.stp = stp;
            this.so = so;
        }
        
        public void run(){
            try{
                
                if(in == null){
                    in = new DataInputStream(s.getInputStream());
                    from = in.readInt();
                    this.so = manager.acceptConnection(s,from);
                }
                byte[] data = null;
                while(true){                      
                    int size = in.readInt();
                    //so.sm.acquire();
                    if (size>=0){
                        data = new byte[size];
                        in.read(data,0,size);
                        //br.ufsc.das.util.Logger.println("ServerThreadPool: Tam recebido de "+from+" = "+data.length);
                        //so.sm.release();                  
                        listener.deliver(new ReceivedMessage(data,from));
                    }
                }
            }catch(Exception e){
                //so.sm.release();         
                try{
                    this.in.close();
                    this.s.close();
                }catch(Exception ignore){
                }
                //br.ufsc.das.util.Logger.println("ServerThreadPool: Erro ao processar msg de " + from);
                e.printStackTrace(System.out);
            }
        }
    }
    
    public class ServerThreadFactory implements ThreadFactory {
        
        public Thread newThread(Runnable r) {
            return new Thread(r);
        }
    }
    
}
