

/*
 * ClientChannel.java
 *
 * Created on 24 de Maio de 2006, 17:14
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package br.ufsc.das.usage;

import br.ufsc.das.usage.nio.SelectorThread;
import br.ufsc.das.util.EventListener;
import br.ufsc.das.util.ReceivedMessage;
import br.ufsc.das.util.Configuration;
import br.ufsc.das.util.Writer;
import java.math.BigInteger;
import java.net.InetSocketAddress;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.util.Hashtable;
import java.util.LinkedList;
import java.util.List;
import java.util.Random;
import java.util.Timer;
import java.util.Vector;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.Semaphore;
import javax.crypto.spec.DHParameterSpec;

/**
 *
 * @author edualchieri
 */
public class ClientChannel implements Writer{
    
    private Hashtable channels = new Hashtable();
    private Semaphore channelSm = new Semaphore(1);
    
    private SelectorThread st;
    private EventListener listener;
    
    private Hashtable pendentInvocations = new Hashtable();
    
    private PrivateKey DHPrikey;
    private PublicKey DHPubkey;
    private Timer timer = new Timer();
    private Configuration conf;
    
    private List<Integer> disconnected = new LinkedList<Integer>();
    
    /** Creates a new instance of ClientChannel */
    public ClientChannel(SelectorThread st, Configuration conf) {
        this.st = st;
        this.conf = conf;
        if(conf.useAuthentication()){
            initKeys();
        }
    }
    
    public void connectionFailed(int remoteId, SendReceivePacket sr){
        timer.schedule(new ConnectionTask(sr),1000);
    }
    
    public void connectionEstablished(int remoteId, SendReceivePacket sr){
        Vector pend = this.removePendent(remoteId);
        if(pend != null){
            for(int i = 0; i < pend.size(); i++){
                sr.write((byte[])pend.elementAt(i));
            }
        }
    }
    
    public void setListener(EventListener l) {
        this.listener = l;
        int autoConnect = conf.getAutoConnectLimit();
        int me = conf.getProcessId();
        for(int i = 0; i < autoConnect && i < me; i++){
            this.connect(i);
        }
    }
    
    public void write(int[] targets, byte[] msg){
        for(int i = 0; i < targets.length; i++){
            if(targets[i] == conf.getProcessId()){
                //br.ufsc.das.util.Logger.println("(ClientChannel) (write) A entregar a mensagem a mim proprio");
                listener.deliver(new ReceivedMessage(msg, conf.getProcessId()));
            }else{
                //br.ufsc.das.util.Logger.println("(ClientChannel) (write) A enviar a mensagem para "+targets[i]);
                //br.ufsc.das.util.Logger.println("(ClientChannel) (write) Antes do connect");
                SendReceivePacket sr = connect(targets[i]);
                //br.ufsc.das.util.Logger.println("(ClientChannel) (write) Depois do connect");
                if(sr != null){
                    //br.ufsc.das.util.Logger.println("(ClientChannel) (write) Antes do isConnected");
                    if(sr.isConnected()){
                        //br.ufsc.das.util.Logger.println("(ClientChannel) (write) Antes do write");
                        sr.write(msg);
                    }else{
                        //br.ufsc.das.util.Logger.println("(ClientChannel) (write) Antes do getPendent");
                        this.getPendent(targets[i]).add(msg);
                    }
                }
            }
        }
        //br.ufsc.das.util.Logger.println("(ClientChannel) (write) Enviei a mensagem para todos");
    }
    
    private SendReceivePacket connect(int target){
        if(disconnected.contains(new Integer(target))){
            return null;
        }
        
        SendReceivePacket ret = (SendReceivePacket) this.channels.get(target);
        if(ret == null){
            try{
                this.channelSm.acquire();
                ret = (SendReceivePacket) this.channels.get(target);
                if(ret == null){
                    if(conf.useAuthentication()){
                        ret = new SendReceiveAuthPacket(target, listener,st, this,conf);
                    }else{
                        ret = new SendReceivePacket(target, listener,st, this,conf);
                    }
                    ret.connect();
                    this.channels.put(target, ret);
                }
                this.channelSm.release();
                
            }catch(Exception e){
                e.printStackTrace();
            }
        }
        return ret;
    }
    
    public void put(int target, SendReceivePacket writer){
        try{
            this.channelSm.acquire();
            if(channels.get(target) == null){
                this.channels.put(target, writer);
            }
            this.channelSm.release();
        }catch(Exception e){
            e.printStackTrace();
        }
    }
    
    public void remove(int target){
        this.channels.remove(target);
        this.disconnected.add(new Integer(target));
    }
    
    private Vector getPendent(int target){
        Vector v = (Vector) this.pendentInvocations.get(target);
        if(v == null){
            v = new Vector();
            this.pendentInvocations.put(target,v);
        }
        return v;
    }
    
    private Vector removePendent(int target){
        return (Vector) this.pendentInvocations.remove(target);
    }
    
    public PublicKey getDHPubKey(){
        return this.DHPubkey;
    }
    
    public PrivateKey getDHPriKey(){
        return this.DHPrikey;
    }
    
    private void initKeys(){
        BigInteger p = conf.getDHP();
        BigInteger g = conf.getDHG();
        int priv = new Random(conf.getProcessId()).nextInt(1000);
        try {
            DHParameterSpec dhSpec = new DHParameterSpec(p, g, priv);
            KeyPairGenerator keyGen = KeyPairGenerator.getInstance("DH");
            keyGen.initialize(dhSpec);
            KeyPair keypair = keyGen.generateKeyPair();
            this.DHPrikey = keypair.getPrivate();
            this.DHPubkey = keypair.getPublic();
        }catch(Exception e){
            e.printStackTrace();
        }
    }
    
}
