

package br.ufsc.das.usage;

import br.ufsc.das.usage.nio.Connector;
import br.ufsc.das.usage.nio.PacketChannel;
import br.ufsc.das.usage.nio.ProtocolDecoderAuth;
import br.ufsc.das.usage.nio.ReceivedPacket;
import br.ufsc.das.usage.nio.SelectorThread;
import br.ufsc.das.usage.nio.ProtocolDecoder;
import br.ufsc.das.util.EventListener;
import br.ufsc.das.util.ReceivedMessage;
import br.ufsc.das.util.Configuration;
import java.io.IOException;
import java.math.BigInteger;
import java.net.InetSocketAddress;
import java.nio.ByteBuffer;
import java.nio.channels.SocketChannel;
import java.security.KeyFactory;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.NoSuchAlgorithmException;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.spec.X509EncodedKeySpec;
import java.util.Random;
import java.util.concurrent.Semaphore;
import javax.crypto.KeyAgreement;
import javax.crypto.KeyGenerator;
import javax.crypto.Mac;
import javax.crypto.SecretKey;
import javax.crypto.spec.DHParameterSpec;
import javax.crypto.spec.SecretKeySpec;


/*
 * ChannelWriter.java
 *
 * Created on 27 de Janeiro de 2006, 15:33
 *
 * To change this template, choose Tools | Options and locate the template under
 * the Source Creation and Management node. Right-click the template and choose
 * Open. You can then make changes to the template in the Source Editor.
 */

/**
 *
 * @author alchieri
 */
public class SendReceiveAuthPacket extends SendReceivePacket{
    
   
    private Mac mac;
    private Semaphore macSem = new Semaphore(1);
    
    
    
    /** Creates a new instance of ChannelWriter */
    /* Para conectar-se*/
    public SendReceiveAuthPacket(int target, EventListener l, SelectorThread st, 
            ClientChannel c, Configuration conf){
        super(target,l,st,c,conf);
    }
    
    /** Creates a new instance of ChannelWriter */
    /* Para receber uma conex�o*/
    public SendReceiveAuthPacket(PacketChannel channel,EventListener l, 
            ClientChannel c, Configuration conf){
        super(channel,l,c,conf);
        this.sm = new Semaphore(0);
        this.connectionEstablished = false;
    }
    
    
    public void write(byte[] msg){
        //br.ufsc.das.util.Logger.println("(SendReceiveAuthPacket) (write) Ah espera do semaforo");
        getMEZone();
        //br.ufsc.das.util.Logger.println("(SendReceiveAuthPacket) (write) Depois do semaforo");
        sendData(msg, getAuthenticationCode(msg), ProtocolDecoderAuth.STX);
    }
    
    private byte[] getAuthenticationCode(byte[] bytesToSend){
         try{
            this.macSem.acquire();
            byte[] ret = mac.doFinal(bytesToSend);
            this.macSem.release();
            return ret;
        }catch(Exception e){
            e.printStackTrace();
        }
        return null;
    }
    
    private void sendData(byte[] data, byte[] aut,byte ctr){
        //ByteBuffer buffer = null;
        ByteBuffer buffer = ByteBuffer.allocate(data.length+aut.length+9); // 9 = 2*int + ctr
        buffer.put(ctr);
        buffer.putInt(data.length);
        buffer.putInt(aut.length);
        buffer.put(data);
        buffer.put(aut);
        buffer.limit(buffer.position());
        buffer.flip();
        this.channel.sendPacket(buffer);
    }
    
    /**
     * A new client connected. Creates a PacketChannel to handle it.
     */
    public void connectionEstablished(Connector connector, SocketChannel sc) {
        try {
            //sc.socket().setReceiveBufferSize(5*1024);
            //sc.socket().setSendBufferSize(5*1024);
            channel = new PacketChannel(sc,st,new ProtocolDecoderAuth(),this);
            channel.resumeReading();
            byte[] pId = new Integer(conf.getProcessId()).toString().getBytes();
            ////br.ufsc.das.util.Logger.println("Conexao estabelecida... "+remoteId);
            sendData(pId, this.clientSide.getDHPubKey().getEncoded(), ProtocolDecoderAuth.SATX);
        } catch (IOException e) {
            e.printStackTrace(System.out);
        }
    }
    
    private boolean checkAuth(ReceivedPacket pckt){
        byte[] aut = getAuthenticationCode(pckt.getData());
        byte[] recAut = pckt.getAuthenticationCode();
        for(int i = 0; i < aut.length; i++){
            if(aut[i] != recAut[i]){
                return false;
            }
        }
        return true;
    }
    
    public void packetArrived(PacketChannel pc, ReceivedPacket pckt) {
        //br.ufsc.das.util.Logger.println("[" + pc.toString() + "] Packet received. Size: "+pckt.getData().length);
        if(pckt.getCtr() == ProtocolDecoder.STX){
            if(checkAuth(pckt)){
                listener.deliver(new ReceivedMessage(pckt.getData(),this.remoteId));
            }else{
                System.err.println("Falha na autentica��o... "+remoteId);
            }
        }else{
            Integer in = new Integer(new String(pckt.getData()));
            int from = in.intValue();
            this.remoteId = from;
            String name = pc.getSocketChannel().socket().getInetAddress().getCanonicalHostName();
            clientSide.put(from,this);
            if(conf.isHostSetted(from)){// esta em hosts...
                String ip = pc.getSocketChannel().socket().getInetAddress().getHostAddress();
                if(!checkServer(from,name, ip)){
                    System.err.println("Conexao nao prevista...");
                    pc.close();
                    clientSide.remove(from);
                    return;
                }
            }
            if(pckt.getCtr() == ProtocolDecoderAuth.SATX){
                //br.ufsc.das.util.Logger.println("Recebeu conexao de "+name);
                this.exchangeSecret(pckt.getAuthenticationCode());
                byte[] pId = new Integer(conf.getProcessId()).toString().getBytes();
                sendData(pId,this.clientSide.getDHPubKey().getEncoded(),ProtocolDecoderAuth.RATX);
            }else{
               System.out.println("Conectou-se a "+name);
                this.exchangeSecret(pckt.getAuthenticationCode());
            }
            connectionEstablished = true;
            this.clientSide.connectionEstablished(this.remoteId, this);
        }
        try {
            pc.resumeReading();
        } catch (Exception e) {
            e.printStackTrace(System.out);
        }
    }
    
    private void exchangeSecret(byte[] otherPartyPubKey){
         try {
            X509EncodedKeySpec x509KeySpec = new X509EncodedKeySpec(otherPartyPubKey);
            KeyFactory keyFact = KeyFactory.getInstance("DH");
            PublicKey publicKey = keyFact.generatePublic(x509KeySpec);
            KeyAgreement ka = KeyAgreement.getInstance("DH");
            ka.init(this.clientSide.getDHPriKey());
            ka.doPhase(publicKey, true);
            byte[] secret = ka.generateSecret();
            SecretKey secretKey = new SecretKeySpec(secret,"HmacMD5");
            this.mac = Mac.getInstance(secretKey.getAlgorithm());
            this.mac.init(secretKey);
        } catch(Exception e){
            e.printStackTrace();
        }
    }
}
