

package br.ufsc.das.usage;

import br.ufsc.das.usage.nio.Connector;
import br.ufsc.das.usage.nio.PacketChannel;
import br.ufsc.das.usage.nio.ReceivedPacket;
import br.ufsc.das.usage.nio.SelectorThread;
import br.ufsc.das.usage.nio.ProtocolDecoder;
import br.ufsc.das.util.EventListener;
import br.ufsc.das.util.ReceivedMessage;
import br.ufsc.das.util.Configuration;
import java.io.IOException;
import java.net.InetSocketAddress;
import java.nio.ByteBuffer;
import java.nio.channels.SocketChannel;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.NoSuchAlgorithmException;
import java.security.PublicKey;
import java.util.concurrent.Semaphore;
import javax.crypto.KeyGenerator;
import javax.crypto.Mac;
import javax.crypto.SecretKey;


/*
 * ChannelWriter.java
 *
 * Created on 27 de Janeiro de 2006, 15:33
 *
 * To change this template, choose Tools | Options and locate the template under
 * the Source Creation and Management node. Right-click the template and choose
 * Open. You can then make changes to the template in the Source Editor.
 */

/**
 *
 * @author alchieri
 */
public class SendReceivePacket {
    protected SelectorThread st;
    protected PacketChannel channel;
    protected EventListener listener;
    protected boolean connectionEstablished = false;
    protected Semaphore sm;
    protected int remoteId = -1;
    protected ClientChannel clientSide;
    protected Configuration conf;  
   
    /** Creates a new instance of ChannelWriter */
    /* Para conectar-se*/
    public SendReceivePacket(int target, EventListener l, SelectorThread st,
            ClientChannel c, Configuration conf){
        this.st = st;
        this.listener = l;
        this.remoteId = target;
        this.conf = conf;
        this.sm = new Semaphore(0);
        this.clientSide = c;
    }
    
    /** Creates a new instance of ChannelWriter */
    /* Para receber uma conex�o*/
    public SendReceivePacket(PacketChannel channel,EventListener l, 
            ClientChannel c, Configuration conf){
        this.sm = new Semaphore(1);
        this.listener = l;
        this.channel = channel;
        this.connectionEstablished = true;
        this.clientSide = c;
        this.conf = conf;
    }
    
    public void connect()  throws Exception{
        InetSocketAddress addr = conf.getRemoteAddress(this.remoteId);
        new Connector(this.st, addr, this).connect();
    }
    
    public boolean isConnected(){
        return this.connectionEstablished;
    }
    
    public void setConnected(){
        this.connectionEstablished = true;
    }
    
    protected void getMEZone(){
        try{
            this.sm.acquire();
        }catch(Exception e){
            e.printStackTrace();
        }
    }
    
    public void write(byte[] msg){
        //br.ufsc.das.util.Logger.println("(SendReceivePacket) (write) Ah espera do semaforo");
        getMEZone();
        //br.ufsc.das.util.Logger.println("(SendReceivePacket) (write) Passou o semaforo");
        sendData(msg,ProtocolDecoder.STX);
    }
    
    private void sendData(byte[] data ,byte ctr){
        //ByteBuffer buffer = null;
        ByteBuffer buffer = ByteBuffer.allocate(data.length+5); // 5 = 1*int + ctr
        buffer.put(ctr);
        buffer.putInt(data.length);
        buffer.put(data);
        buffer.limit(buffer.position());
        buffer.flip();
        this.channel.sendPacket(buffer);
    }
    
    /**
     * A new client connected. Creates a PacketChannel to handle it.
     */
    public void connectionEstablished(Connector connector, SocketChannel sc) {
        try {
            sc.socket().setReceiveBufferSize(200*1024);
            sc.socket().setSendBufferSize(200*1024);
            channel = new PacketChannel(sc,st,new ProtocolDecoder(),this);
            channel.resumeReading();
            byte[] pId = new Integer(conf.getProcessId()).toString().getBytes();
            sendData(pId,ProtocolDecoder.ATX);
            
            String name = sc.socket().getInetAddress().getHostName();
            System.out.println("Conectou-se a "+name);
            this.connectionEstablished = true;
            this.clientSide.connectionEstablished(this.remoteId, this);
          } catch (IOException e) {
            e.printStackTrace();
        }
    }
    
    
    
    public void connectionFailed(Connector connector, Exception cause) {
        this.clientSide.connectionFailed(this.remoteId, this);
     }
    
    
    public void packetArrived(PacketChannel pc, ReceivedPacket pckt) {
        //br.ufsc.das.util.Logger.println("[" + pc.toString() + "] Packet received. Size: "+pckt.getData().length);
        if(pckt.getCtr() == ProtocolDecoder.STX){
             listener.deliver( new ReceivedMessage(pckt.getData(),this.remoteId));
        }else{
            Integer in = new Integer(new String(pckt.getData()));
            int from = in.intValue();
            this.remoteId = from;
            clientSide.put(from,this);
            if(conf.isHostSetted(from)){ //esta em hosts...
                String name = pc.getSocketChannel().socket().getInetAddress().getHostName();
                String ip = pc.getSocketChannel().socket().getInetAddress().getHostAddress();
                if(checkServer(from,name, ip)){
                    //br.ufsc.das.util.Logger.println("Recebeu conexao de "+name);
                }else{
                    System.err.println("Conexao nao prevista...");
                    pc.close();
                    clientSide.remove(from);
                    return;
                }
            }
        }
        try {
            pc.resumeReading();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    protected boolean checkServer(int from, String name, String ip){
        String localConf = conf.getHost(from);
        if(!(name.equalsIgnoreCase(localConf) || ip.equalsIgnoreCase(localConf))){
            int i = name.indexOf(".");
            if(!name.substring(0,i).equalsIgnoreCase(localConf)){
                return false;
            }
        }
        return true;
    }
    
    public void socketException(PacketChannel pc, Exception ex) {
        ////br.ufsc.das.util.Logger.println("[" + remoteId + "] Error: " + ex.getMessage());
        this.sm.release();
    }
    
    public void socketDisconnected(PacketChannel pc) {
        String name = pc.getSocketChannel().socket().getInetAddress().getCanonicalHostName();
        //br.ufsc.das.util.Logger.println("[" + name  + "("+remoteId+")] Disconnected.");
        this.clientSide.remove(remoteId);
    }
    
    
    public void packetSent(PacketChannel pc, ByteBuffer pckt) {
        this.sm.release();
    }
}
