/*
 * PacketChannel.java
 *
 * Created on 24 de Maio de 2006, 11:34
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package br.ufsc.das.usage.nio;

import br.ufsc.das.usage.SendReceivePacket;
import br.ufsc.das.usage.nio.ProtocolDecoder;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.SelectionKey;
import java.nio.channels.SocketChannel;


final public class PacketChannel {
    
    protected final SelectorThread selector;
    private final SocketChannel sc;
    private ByteBuffer inBuffer;
    private ByteBuffer outBuffer = null;
    private final ProtocolDecoder protocolDecoder;
    private SendReceivePacket listener;
    
    public PacketChannel(SocketChannel socketChannel,SelectorThread selector,
            ProtocolDecoder protocolDecoder,SendReceivePacket listener) throws IOException {
        
        this.selector = selector;
        this.protocolDecoder = protocolDecoder;
        this.sc = socketChannel;
        this.listener = listener;
        socketChannel.socket().setTcpNoDelay(true);
        inBuffer = ByteBuffer.allocateDirect(sc.socket().getReceiveBufferSize());
        inBuffer.position(inBuffer.capacity());
        selector.registerChannelNow(sc, SelectionKey.OP_READ,  this);
    }
    
    public void setListener(SendReceivePacket listener){
        this.listener = listener;
    }
    
    public void resumeReading() throws IOException {
        processInBuffer();
    }
    
    
    public void close() {
        try {
            sc.close();
        } catch (IOException ignore) {
            
        }
    }
    
    public void handleRead() {
        try {
            int readBytes = sc.read(inBuffer);
            if (readBytes == -1) {
                close();
                listener.socketDisconnected(this);
                return;
            }
            if (readBytes == 0) {
                reactivateReading();
                return;
            }
            inBuffer.flip();
            processInBuffer();
        } catch (IOException ex) {
            listener.socketException(this, ex);
            close();
        }
    }
    
    private void processInBuffer() throws IOException {
        ReceivedPacket packet = null;
        try{
            packet = protocolDecoder.decode(inBuffer);
        }catch(Exception e){
            
        }
        
        if (packet == null) {
            inBuffer.clear();
            reactivateReading();
        } else {
            listener.packetArrived(this, packet);
        }
    }

    public void disableReading() throws IOException {
        selector.removeChannelInterestNow(sc, SelectionKey.OP_READ);
    }
    
    private void reactivateReading() throws IOException {
        selector.addChannelInterestNow(sc, SelectionKey.OP_READ);
    }
    

    public void sendPacket(ByteBuffer packet) {
        outBuffer = packet;
        handleWrite();
    }
    

    private void requestWrite() throws IOException {
        selector.addChannelInterestLater(sc, SelectionKey.OP_WRITE,null);
    }
    
    public void handleWrite() {
        try {
            sc.write(outBuffer);
            if (outBuffer.hasRemaining()) {
                requestWrite();
            } else {
                ByteBuffer sentPacket = outBuffer;
                outBuffer = null;
                listener.packetSent(this, sentPacket);
            }
        } catch (IOException ioe) {
            //ioe.printStackTrace(System.out);
            listener.socketException(this, ioe);
            close();
        }
    }
    
    public SocketChannel getSocketChannel() {
        return sc;
    }
    
    public String toString() {
        return Integer.toString(sc.socket().getLocalPort());
    }
}